// Copyright 2010-2012, Google Inc.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
//     * Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above
// copyright notice, this list of conditions and the following disclaimer
// in the documentation and/or other materials provided with the
// distribution.
//     * Neither the name of Google Inc. nor the names of its
// contributors may be used to endorse or promote products derived from
// this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include "converter/connector.h"

#include "base/thread.h"

namespace mozc {
namespace {

// ConverterFactory
ConnectorInterface *g_connector = NULL;

#ifdef MOZC_USE_SEPARATE_CONNECTION_DATA
const char *kConnectionData_data = NULL;
const size_t kConnectionData_size = 0;
#else
#include "converter/embedded_connection_data.h"
#endif  // MOZC_USE_SEPARATE_CONNECTION_DATA

const int kCacheSize = 1024;

char* g_connection_data_address = const_cast<char*>(kConnectionData_data);
int g_connection_data_size = kConnectionData_size;
}  // namespace

namespace connector_internal {
// Use Thread Local Storage for Cache of the Connector.
TLS_KEYWORD bool g_cache_initialized = false;
TLS_KEYWORD int  g_cache_key[kCacheSize];
TLS_KEYWORD int  g_cache_value[kCacheSize];
}  // namespace connector_internal

Connector::Connector() : ConnectorBase(
    g_connection_data_address,
    g_connection_data_size,
    &connector_internal::g_cache_initialized,
    connector_internal::g_cache_key,
    connector_internal::g_cache_value,
    kCacheSize) {}

void ConnectorFactory::SetConnector(ConnectorInterface *connector) {
  g_connector = connector;
}

ConnectorInterface *ConnectorFactory::GetConnector() {
  if (g_connector == NULL) {
    if (!g_connection_data_address || !g_connection_data_size) {
      LOG(FATAL) << "Connector data address/size is/are not set yet.";
      CHECK(false);
    }
    return Singleton<Connector>::get();
  } else {
    return g_connector;
  }
}

void ConnectorFactory::SetConnectionData(void *address, size_t size) {
  g_connection_data_address = reinterpret_cast<char*>(address);
  g_connection_data_size = size;
}

}  // namespace mozc
